package cn.ibizlab.core.dict.service.impl;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.math.BigInteger;
import java.util.Collection;
import java.util.Objects;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.transaction.annotation.Transactional;
import cn.ibizlab.core.dict.domain.IBZDictItem;
import cn.ibizlab.core.dict.filter.IBZDictItemSearchContext;
import cn.ibizlab.core.dict.service.IIBZDictItemService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.dict.mapper.IBZDictItemMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.ObjectUtils;


/**
 * 实体[字典项目] 服务对象接口实现
 */
@Slf4j
@Service
public class IBZDictItemServiceImpl extends ServiceImpl<IBZDictItemMapper, IBZDictItem> implements IIBZDictItemService {


    @Override
    @Transactional
    public IBZDictItem get(String key) {
        IBZDictItem et = getById(key);
        if(et==null){
            et=new IBZDictItem();
            et.setItemid(key);
        }
        return et;
    }

    @Override
    public IBZDictItem getDraft(IBZDictItem et) {
        return et;
    }

    @Override
    @Transactional
    public boolean save(IBZDictItem et) {
        return saveOrUpdate(et) ;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(IBZDictItem et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<IBZDictItem> list, int batchSize) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(IBZDictItem et) {
        return (!ObjectUtils.isEmpty(et.getItemid()))&&(!Objects.isNull(this.getById(et.getItemid())));
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList, int batchSize) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean create(IBZDictItem et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getItemid()),et);
        return true ;
    }

    @Override
    public void createBatch(List<IBZDictItem> list, int batchSize) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZDictItem et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibzdictitemid",et.getItemid())))
            return false;
        CachedBeanCopier.copy(get(et.getItemid()),et);
        return true ;
    }

    @Override
    public void updateBatch(List<IBZDictItem> list, int batchSize) {
        updateBatchById(list,batchSize);
    }


	@Override
	public List<IBZDictItem> selectByDictid(String dictid) {
	    return baseMapper.selectByDictid(dictid);
	}

	@Override
	public void saveByDictid(String dictid,List<IBZDictItem> list) {
	    if(list==null)
            return;
        Set<String> delIds=new HashSet<String>();
        List<IBZDictItem> _update=new ArrayList<IBZDictItem>();
        List<IBZDictItem> _create=new ArrayList<IBZDictItem>();
        for(IBZDictItem before:selectByDictid(dictid)){
            delIds.add(before.getItemid());
        }
        for(IBZDictItem sub:list) {
            sub.setDictid(dictid);
            if(ObjectUtils.isEmpty(sub.getItemid()))
                sub.setItemid((String)sub.getDefaultKey(true));
            if(delIds.contains(sub.getItemid())) {
                delIds.remove(sub.getItemid());
                _update.add(sub);
            }
            else
                _create.add(sub);
        }
        if(_update.size()>0)
            this.updateBatch(_update,500);
        if(_create.size()>0)
            this.createBatch(_create,500);
        if(delIds.size()>0)
            this.removeBatch(delIds,500);
	}

	@Override
	public void removeByDictid(String dictid) {
	    this.remove(new QueryWrapper<IBZDictItem>().eq("ibzdictid",dictid));
	}


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZDictItem> searchDefault(IBZDictItemSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZDictItem> pages=baseMapper.searchDefault(context.getPage(),context,context.getSelectCond());
        return new PageImpl<IBZDictItem>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


}


