package cn.ibizlab.core.dict.service.impl;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.math.BigInteger;
import java.util.Collection;
import java.util.Objects;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.transaction.annotation.Transactional;
import cn.ibizlab.core.dict.domain.IBZDict;
import cn.ibizlab.core.dict.filter.IBZDictSearchContext;
import cn.ibizlab.core.dict.service.IIBZDictService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.dict.mapper.IBZDictMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.ObjectUtils;


/**
 * 实体[数据字典] 服务对象接口实现
 */
@Slf4j
@Service
public class IBZDictServiceImpl extends ServiceImpl<IBZDictMapper, IBZDict> implements IIBZDictService {

    @Autowired
    private cn.ibizlab.core.dict.service.IIBZDictItemService ibzdictitemService;

    @Override
    @Transactional
    public IBZDict get(String key) {
        IBZDict et = getById(key);
        if(et==null){
            et=new IBZDict();
            et.setDictid(key);
        }
        return et;
    }

    @Override
    @Transactional
    public boolean create(IBZDict et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getDictid()),et);
        return true ;
    }

    @Override
    public void createBatch(List<IBZDict> list, int batchSize) {
        saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(IBZDict et) {
        return (!ObjectUtils.isEmpty(et.getDictid()))&&(!Objects.isNull(this.getById(et.getDictid())));
    }

    @Override
    public IBZDict getDraft(IBZDict et) {
        return et;
    }

    @Override
    @Transactional
    public boolean update(IBZDict et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibzdictid",et.getDictid())))
            return false;
        CachedBeanCopier.copy(get(et.getDictid()),et);
        return true ;
    }

    @Override
    public void updateBatch(List<IBZDict> list, int batchSize) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibzdictitemService.removeByDictid(key) ;
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList, int batchSize) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(IBZDict et) {
        return saveOrUpdate(et) ;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(IBZDict et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<IBZDict> list, int batchSize) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZDict> searchDefault(IBZDictSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZDict> pages=baseMapper.searchDefault(context.getPage(),context,context.getSelectCond());
        return new PageImpl<IBZDict>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


}


